function [sol, info] = tomgm(ODE,tspan,y0in,options,varargin)
%TOMGM   
%
%   SOL = TOMGM(ODEFUN,TSPAN,y0) integrates a system of ordinary
%   differential equations of the form y' = f(x,y) 
%   TOMGM produces an approximate solution in the output mesh points
%       SOL.x  -- mesh selected by TOM
%       SOL.y  -- approximation to y(x) at the mesh points of SOL.x
%       SOL.solver -- 'tom'  | 'etr'


%   SOL = TOMGM(ODEFUN,TSPAN,y0,OPTIONS) solves as above with default
%   parameters replaced by values in OPTIONS, a structure created with the
%   TOMGSET function. To reduce the run time greatly, use OPTIONS to supply 
%   a function for evaluating the Jacobian and/or vectorize ODEFUN. 
%   See TOMGSET for details.
%
%   SOL = TOM(ODEFUN,TSPAN,y0,OPTIONS,P1,P2...) passes constant, known
%   parameters P1, P2... to the function ODEFUN, and to all 
%   functions specified in OPTIONS.   
%   
%
%   See  TOMGSET, TOMGGET.
% 
%
%
%  Authors:
%
%  Francesca Mazzia
%  Dipartimento  di Matematica
%  via Orabona 4
%  70125 Bari, Italy
%  e-mail : mazzia@dm.uniba.it
%  
%  Raffaella Pavani
%  Dipartimento di Matematica
%  Politecnico di Milano
%  Piazza Leonardo da Vinci 22
%  20133 Milano, Italy
%  e-mail : raffaella.pavani@polimi.it
%
%  references :
%
% L.Brugnano, D.Trigiante,
% Solving ODEs by Linear Multistep Formulae: Initial and Boundary
% Value Methods - Gordon and Breach. (1998)
%
%
% 



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% check input parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if nargin <3
    error('Not enough input arguments')
end


ExtraArgs = varargin;    


if isempty(tspan) | (length(tspan) ~= 2)
    error(['''' inputname(3) 'tspan'' must contain t the two end points'])
end


if any( diff(tspan) <= 0 )
    error (['The entries in ''' inputname(3) ...
            'tspan'' must strictly increase']);
end



if isempty(y0in)
    error(['No initial guess provided in ''' inputname(3) 'y0'''])
end


m = size(y0in,1);       % size of the ODE system


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get options and set the defaults
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5%%%%%%%%%
if nargin<4
    options = [];
end
order  = tomgget(options,'Order',6);
met = tomgget(options,'Method',1);


outstep = tomgget(options,'OutputSteps',1);
if (size(outstep) ~= [1 1])
    error('The option ''OutputSteps'' must contain a single number');
end
outflag = 0;
if (outstep > 0)
    outflag = 1;
else
    outstep = 1;
end
addpoint = 0;
outpoint = 0;


if (rem(order,2) ~= 0 )
    warning('The order must be even, the order has been set to 10');
    order =  10;
end  
if met == 1 
    k= order/2;
else
    k = order-1;
end 



INTTOT = tspan(2)-tspan(1);
% number of mesh points

% dblk is the number if interior subintervals
% dblk-1 is the number of interior points
% when dblk=order-1, then the minimum mesh per stepsize (i.e. TINT) is used


%dblk=order-1,
dblk=order+1;

tfinal = tspan(2);
t0 = tspan(1);
TINT = tomgget(options,'StepSize');
if (isempty(TINT))
    nsteps = tomgget(options,'NumSteps');
    if (isempty(nsteps))
        TINT = min(1e-2,(tfinal-t0)/2);
        fprintf('Warning: No initial step size provided, using h=1e-2 instead.\n');
    else
        TINT = (tfinal - t0)/nsteps;
    end
end
if ((~isa(TINT,'double')) | (size(TINT) ~= [1 1]))
    error('The option ''StepSize'' must contain a single number');
end

%  Set the fixed interior stepsize

h=TINT/dblk;
nsteps = ceil((tfinal-t0)/TINT);

% Get options and set the defaults

% analytical Jacobians
ODEjac = tomgget(options,'FJacobian');
rtol_newt = tomgget(options,'RelTolNewt',10*eps);
fixedpoint = strcmp(tomgget(options,'FixedPoint','on'),'on');
printstatsfin = strcmp(tomgget(options,'Stats','off'),'on');
printstats    = strcmp(tomgget(options,'Stats_step','off'),'on');
xyVectorized = strcmp(tomgget(options,'Vectorized','off'),'on');
if xyVectorized     % vectorized wrt x, y
    vectorized = 2;   % input to numjac
else
    vectorized = 0;
end


FAC = [];
threshval = 1e-10;
threshold = threshval(ones(m,1));
maxitnl   = tomgget(options,'Itnlmax',50);      % maximum number of nonlinear iteration


if nargout > 0
  outfun = tomgget(options,'OutputFcn');
else
  outfun = tomgget(options,'OutputFcn','odeplot');
end
if isempty(outfun)
  haveoutfun = 0;
else
  haveoutfun = 1;
  outputsel = tomgget(options,'OutputSel',1:m);
end
% Initialize the output function.
if haveoutfun
  feval(outfun,[t0 tfinal],y0in(outputsel),'init');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% setting initial values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

itnl      = 0;
nODEeval = 0;


switch met
case 2
    solver = 'gam';  
    ord   = k + 1;
case 1 
    solver = 'tom';          
    ord   = 2*k;   
end


if (nargout > 0)
    chunk = max(ceil(128 / (m) ),1);
    sol.x = zeros(chunk,1);
    sol.y = zeros(m,chunk);
end
t0     = tspan(1);
tfin   = t0+TINT;
y0last = y0in;
f0last = feval( ODE, t0,y0last, ExtraArgs{:});
dblk1 = dblk+1;
[Balfa,Bbeta,a0,b0,AB,Akron,hstep] = calcola_coeff( met, k, dblk, m, h);
hsts=cumsum(hstep);
tt0=[t0 t0+hsts'*TINT];

%%%%%%
%%  starting cycle for computing the solution
%%%%%%

for jstep = 1:nsteps
    tt0=[t0 t0+hsts'*TINT];
    y      = -a0*y0last'+b0*f0last';
    y0init = y';
    y = y + AB*(f0last*ones(1,dblk))';
    y = [y0last y'];
    tt   = tt0';
    itnl = 0;
     vainl  = 1;
    
    % nonlinear iteration 
    
    while vainl  
        
        if xyVectorized 
            ODE_F = feval(ODE,tt(:)',y,ExtraArgs{:});
            nODEeval=nODEeval+1;            
        else
            for i = 1:dblk1
               ODE_F(:,i) = feval( ODE, tt(i),y(:,i), ExtraArgs{:});
            end
            nODEeval=nODEeval+dblk1;
        end
        
        if ~fixedpoint 
             
            nuovo_jac = itnl == 0;
            
            if nuovo_jac 
                
                if isempty(ODEjac) 
                    [ JAC_F,FAC,dummy,dummy,nFcalls]= numjac(ODE,tt(1),y(:,1),ODE_F(:,1),threshold,FAC,vectorized,[],[],ExtraArgs{:});
                    nODEeval = nODEeval+nFcalls;                   
                else
                    JAC_F = feval(ODEjac,tt(1),y(:,1),ExtraArgs{:}); 
                    
                end
                I=speye(m);
                JAC = Akron(:,m+1:m*(dblk+1))-kron(Bbeta(1:dblk,2:dblk+1),JAC_F);  
                JAC=sparse(JAC);
                [L,U,P,Q]=lu(JAC) ; 
                [Lf,Uf,Pf,Qf]=lu(flipud(fliplr(JAC)));
                Pff =flipud(fliplr(Pf));
                Qff =flipud(fliplr(Qf));
                Lff = flipud(fliplr(Lf));
                Uff = flipud(fliplr(Uf));
            end
            
            
            F = reshape((-Balfa(1:dblk,:)*y'+Bbeta(1:dblk,:)*ODE_F')',m*(dblk),1);   
        
            if max(isinf(JAC))| max(isnan(JAC))
                error('The matrix is singular')
            end  
            
            chsi_in = Q*(U\(L\(P*F)));
            chsi = reshape(chsi_in,m,dblk);       
            y(:,2:end) = y(:,2:end)+chsi;

        else
            F = reshape((-Balfa(1:dblk,:)*y'+Bbeta(1:dblk,:)*ODE_F')',m*(dblk),1);% Aggiunto da Bruno
            chsi = y(:,2:end);
            y(:,2:end) = y0init + (AB*ODE_F(:,2:end)')';
            chsi = chsi-y(:,2:end);
            
        end 
        
        if max(isinf(y))| max(isnan(y))
            error('The solution is INF or NaN ')
        end  
        
        norm_in = sqrt(sum(sum(((chsi)./max(0.1,abs(y(:,1))*ones(1,dblk))).^2)) / (dblk*m));        
        norm_F=norm(F,2);       
        stop_crit  = norm_in <= rtol_newt & norm_F<10*eps;
               
        if printstats
            fprintf('===> Non linear iteration  = %g , newterr %g, \n',itnl+1, norm_in)          
        end
        
            if  itnl > maxitnl 
            
            if (norm_in <= 1e5*rtol_newt)
                msg = sprintf('Convergence failed after %d iterations.\nObtained error = %0.5g, continuing anyway...',maxitnl,norm_in);
                warning(msg);
                
                stop_crit = 1;
            else
                msg = sprintf('Convergence failed after %d iterations.\nObtained error = %0.5g, stopping here.',maxitnl,norm_in);
                warning(msg);
                      
            sol.err = 1;  % unable to meet the tolerance  a number of nonlinear iteration greather than the maximum is required            
            info.nODEeval = nODEeval;    
            info.itnl = itnl+1;
            
            return
           end
            
        end 
        
        if stop_crit
                      
            addpoint = 0;
            outpoint = outpoint + 1; 
            if (outpoint >= outstep) & (outflag)      
                outpoint = 0;
                addpoint = 1;
            end
            if nargout > 0
                if  jstep==1
                    sol.x(1,1) = tt(1);
                    sol.y(:,1) = y(:,1);
                    sol.err = 0;
                    npoint = 1;
                    info.nODEeval = nODEeval;   
                end
                if outflag  & addpoint
                    sol.x(npoint+1:npoint+1,:) =  tt(dblk+1:dblk+1)';
                    sol.y(:,npoint+1:npoint+1) =  y(:,dblk+1:dblk+1);
                    npoint=npoint+1;
                    sol.err = 0;
                    info.nODEeval = nODEeval;   
                end
          
                
            if haveoutfun & addpoint
                if feval(outfun,sol.x(npoint),sol.y(outputsel,npoint),'') == 1
                return;
                             end
             end
           elseif  haveoutfun & addpoint
                        
                if feval(outfun,sol.x(npoint),sol.y(outputsel,npoint),'') == 1
                return;
                             end
           end
                          
            y0last = y(:,dblk+1);
            f0last = ODE_F(:,dblk+1);
            t0 = tt(dblk+1);
            tfin = t0+TINT; 
            
            break
        end
        
        
        if ( itnl <= maxitnl)
            itnl = itnl+1;            
            tt0 = tt;
            y0 = y;
        end
       
        
    end
    
end
sol.x = sol.x(1:npoint);
sol.y = sol.y(:,1:npoint);
if printstatsfin       
    fprintf('#### The solution was obtained at the step %g. \n',jstep)
    fprintf('     There were %g calls to the ODE function. \n',nODEeval); 
  
    
end
return    




function [Balfa,Bbeta,a0,b0,AB,Akron,hstep] = calcola_coeff( met, k, dblk, m, h)

[Are, Bre, tvar, hstep ] = tommatrin(k,dblk,met);
Bbeta =Bre;
Balfa = Are;
Bbeta = h*Bbeta; 
a0 = Balfa(1:end,2:end)\Balfa(1:end,1);
b0 = Balfa(1:end,2:end)\Bbeta(1:end,1);
AB = full(Balfa(1:end,2:end)\Bbeta(1:end,2:end));
Akron = kron(Balfa(1:end,:),speye(m));

return   
                             
               

function [Are, Bre, tvar, hstep ] = tommatrin(k,dblk,method) 
%  [Are,Bre,tvar,hstep]=tommatrin(5,15,'TOM');
% matrici dei coefficienti estese calcolate per un intervallo [0,1] con passo
% variabile e scalate in modo da risolvere il problema Are y - h*Bre f = 0
% con h =  1/dblk 
% hstep contiene i passi utilizzati
% method = TOM o GAM

t0 = 0;
h = 1; % intervallo iniziale
var = 1; % passo variabile

% con dblk = k+2 i metodi sembrano diventare A-stabili 
% con dblk = k e k+1 la regione di stabilita' ha piccole regioni
% nel semipiano negativo.
%

k1 = fix(k/2)+1;
nstep = 0;
 if method==1
   dblkin = 2*k-1;
 else
   dblkin=k;
 end

[tvariniz,w,P]= lglnodes(dblkin); % nodi di Chebyshec-Gauss-Lobatto
tvariniz=sort(tvariniz);  
tvariniz = (1/2*(1+tvariniz));
 if var 
  tvar=estendinodi(tvariniz,dblk);
  tvar=tvar/tvar(end);
  hstep = diff(tvar)';
 else
   tvar = t0+[0:dblk]*h/dblk; % nodi costanti
   tvar=tvar;
   hstep = diff(tvar');
 end
hstep=(hstep+flipud(fliplr(hstep)))/2;
tvar = cumsum([t0;hstep])';

k1 = fix(k/2)+1;  k2=k-k1; 
 if  method == 1
     kk=2*k-1;
     kk1=fix((2*k-1)/2)+1;
     kk2=2*k-1-kk1;
 else
     kk=k;
     kk1=k1;
     kk2=k2;
 end
 for n=0:k1-1
     [ro(:,n+1),si(:,n+1)] = rosi_gam(kk,n,hstep,tvar);
 end
 if method==1 & 0
    [ros,sis] = rosi_gam(kk,n+1,hstep,tvar);
    for n=0:k1-2
        ro(:,n+1)=ro(:,n+1)-ro(:,n+2)*(si(kk+1,n+1)/si(kk+1,n+2));
        si(:,n+1)=si(:,n+1)-si(:,n+2)*(si(kk+1,n+1)/si(kk+1,n+2));       
    end
    n=k1-1;
    ro(:,n+1)=ro(:,n+1)-ros*(si(kk+1,n+1)/sis(kk+1));
    si(:,n+1)=si(:,n+1)-sis*(si(kk+1,n+1)/sis(kk+1));       
 end
i=1;
 for n=k1:dblk-k2
    if method==1 
       [ro(1:k+1,n+1),si(1:k+1,n+1)] = rosi_tom(k,k1,hstep(i:end),tvar(i:end));
    else
       [ro(:,n+1),si(:,n+1)] = rosi_gam(kk,kk1,hstep(i:end),tvar(i:end));
    end
    i=i+1;
 end
j=k2-1;
 for n=dblk-k2+1:dblk 
     [ro(:,n+1),si(:,n+1)] = rosi_gam(kk,kk-j,hstep(dblk-kk+1:end),tvar(dblk-kk+1:end));
     j=j-1;
 end
 if method==1 & 0
     [ros,sis] = rosi_gam(kk,kk-k2,hstep(dblk-kk+1:end),tvar(dblk-kk+1:end));
     for n=dblk-k2+2:dblk
        ro(:,n+1)=ro(:,n+1)-ro(:,n)*(si(1,n+1)/si(1,n));
        si(:,n+1)=si(:,n+1)-si(:,n)*(si(1,n+1)/si(1,n));       
     end
     n=dblk-k2+1;
     ro(:,n+1)=ro(:,n+1)-ros*(si(1,n+1)/sis(1));
     si(:,n+1)=si(:,n+1)-sis*(si(1,n+1)/sis(1));    
 end
r = dblk;
s= 1;
hstep = [hstep(1) ; hstep];
 for n=1:r+1
  si(:,n) = si(:,n)*hstep(n)/(h/dblk);  
 end
 for n=1:k1
  irig = (n-1)*s+1:n*s;
  for i=1:kk+1
    icol = (i-1)*s+1:i*s;
    B(irig,icol) = si(i,n); 
    D(irig,icol) = ro(i,n); 
  end
  B(irig,irig) = B(irig,irig); %-c(irig,1:s);
 end

% righe corrispondenti al metodo principale

 for n=k1+1:r-k2+1   
  irig = (n-1)*s+1:n*s;
  for i=1:k+1
    j=n+i-k1-1;
    icol = (j-1)*s+1:j*s;
    B(irig,icol) = si(i,n); %*sparse( E(icol,1:s) ) ;
    D(irig,icol) = ro(i,n);
  end
  B(irig,irig) = B(irig,irig); %-c(irig,1:s);
 end
 
% righe corrispondenti ai metodi finali

 for n=r-k2+2:r+1
   irig = (n-1)*s+1:n*s;
   for i=1:kk+1
     j=r+i-kk;
     icol = (j-1)*s+1:j*s;
     B(irig,icol)= si(i,n); 
     D(irig,icol) = ro(i,n);
   end
   B(irig,irig) = B(irig,irig);
 end

B(2:end,1:end)= (flipud(fliplr( B(2:end,1:end)))+B(2:end,1:end))/2;
D(2:end,1:end)= -(flipud(fliplr(D(2:end,1:end)))-D(2:end,1:end))/2;
Dr = full(D(2:end,2:end));
Dre = full(D(2:end,1:end));
Are = Dre;
Bre = full(B(2:end,1:end));
Br = full(B(2:end,2:end));
b0 = full(B(2:end,1));
d0 = full(D(2:end,1));
hstep =hstep(2:end);
clear ev
step = linspace(0,2*pi,100);

 for i=1:100
    w = exp(eps+sqrt(-1)*step(i));
    Ae = Dr;
    Be = Br;
    Ae0 = zeros(size(Dr)); 
    Ae0(:,end)=d0;
    Be0 = zeros(size(Dr)); 
    Be0(:,end)=b0;
     if abs(w-1)> 1e-5 
        ev(:,i) = eig( -inv( (h/dblk)*Be(:,1:end)+ (h/dblk)*Be0(:,1:end)/w)*(Ae+Ae0/w));
     end
 end

return



function [ro,si] = rosi_tom( k,j,h, tvar )

%
% Coefficienti del TOM a K passi, K = 2*NU-1.
% Il metodo va usato con NU condizioni iniziali.
% Coefficienti dei polinomi per potenze crescenti.
%
                        
tt(j:-1:1)=-(cumsum((h(j:-1:1))));
tt(j+1)=0;
tt(j+2:k+1)=cumsum(h(j+1:k));
tt=tt/h(j);
a  = cvsolven( tt, [zeros(2*k+1,1); 1],tvar'); 
si = -a(k+2:2*k+2); ro = a(1:k+1); ssi=sum(si); ro = ro/ssi; si = si/ssi;
[ro1,si1]=vbnewtom(k,tvar(1:k+1));
ro = -ro1; si=-si1;

return



function x = vsolven( a, b ,ji,tn)

%
%     f = vsolve( x, b )      Risolve il sistema lineare W(x) f = b,
%                             dove  W(x)  e' la Vandermonde definita
%                             dagli elementi del vettore x.


k = length(a);
V=ones(k);
V1=V;
 for i=1:k
    t(i)=(a(i)-a(1))/(a(k)-a(1));
 end    

 for j=2:k
    V(j,:)=a.*V(j-1,:);
    V1(j,:)=t.*V1(j-1,:);
 end
T=zeros(k,k);
 for i=1:k
    for j=i:k
        %T(i,j) = nchoosek(k-1,i-1)*nchoosek(k-i,j-i)*(-1)^(j-i);
         T(i,j) = nchoosek(k-1,j-1)*nchoosek(j-1,i-1)*(-1)^(j-i);
    end
 end
b1=zeros(k,1);
 if ji > 0
   b1(ji)=-1;
   b1(ji+1)=1;
 else
   b1(ji+1)=1;
   b1(ji+2)=-1;
 end
b1=T*diag( (a(k)-a(1))./[1:k])*V1*diag(t)*b1;
V1=T*V1;
x = V1\b1;
x=V\b;

return



function x = cvsolven( a, b, tn )
%
%     X = CVSOLVE( A, B )     Risolve il sistema di Vandermonde confluente
%                 -                                 -
%                 |   1   ...  1       0 ...   0    |
%                 |  a0   ... ak       1 ...   1    |
%                 |   .........      2a0 ... 2ak    |  X = B,   m = k-1.
%                 |   .........        .......      |
%                 |  a0^k ... ak^k  ka0^m ... kak^m |
%                 -                                 -
%

k = length(a);
n=2*k;
V=ones(n);
  for i=1:k
     t(i)=(tn(i)-tn(1))/(tn(k)-tn(1));
  end    
V(2,1:k)=a.*V(1,1:k);
V(1,k+1:n)=  zeros(1,k);
V1=ones(n);
V1(2,1:k)=t.*V1(1,1:k);
V1(1,k+1:n)=  zeros(1,k);
H=zeros(n-1);
  for j=3:n
    V(j,1:k)=a.*V(j-1,1:k);
    V1(j,1:k)=t.*V1(j-1,1:k);
    V(j,k+1:n)= (j-1)*a.*V(j-1,k+1:n)/(j-2);
  end
 for i=2:n-1
    H(i,i-1)=i-1;
 end
T=zeros(n-1,n-1);
  for i=1:n-1
    for j=i:n-1
       T(i,j) = nchoosek(n-2,j-1)*nchoosek(j-1,i-1)*(-1)^(j-i);
    end
  end
H=T*H;
H = H/(a(k)-a(1));
V1(n,1:k)=zeros(1,k);
V1(n,k+1:n)=ones(1,k);
V1(1:n-1,k+1:n)= H*V1(1:n-1,1:k);
V1(1:n-1,1:k)=T*V1(1:n-1,1:k);
x = V1\b;

return



function [ro,si] = rosi_gam( k, j,h ,tvar);
%
% Polinomi RO e SI del metodo di Adams generalizzato a K passi e j condizioni
% iniziali. I metodi di Adams-Moulton hanno J=K.
% 
% Written by:  Luigi BRUGNANO

 if j<0 | j>k, 
    error( 'J<K!' ),
 end
b  = zeros(k+1,1); 
s = 1; 
b(1) = 1;
 for i = 2:k+1, 
     s=-s; 
     b(i) = s/i; 
 end
 for i=1:j
   tt(i) = -sum(h(i:j))/h(j);
 end  
 if j>0
   tt(j+1)=0;
   for i=j+1:k
        tt(i+1)= sum( h(j+1:i))/h(j);
   end   
 else
     tt(j+1)=0;
     for i=j+1:k
        tt(i+1)= sum( h(j+1:i))/h(j+1);
     end   
 end
k1=fix(k/2);
si = vsolven( tt , b(:),j ,tvar');
 if j>0
   si1=vbnew_gam(k,tvar(1:k+1),j-k1-1);
   si=si1;
 end
ro = zeros( k+1, 1 ); 
 if j>0
   ro(j:j+1) = [-1; 1];
 else
   ro(j+1:j+2) = [1; -1];
 end 
 if nargout==1, 
     ro=si; 
 end
si = sparse(si); 
ro = sparse(ro);

return



function f = vsolve( x, b )
%
%     f = vsolve( x, b )      Risolve il sistema lineare W(x) f = b,
%                             dove  W(x)  e' la Vandermonde definita
%                             dagli elementi del vettore x.
%

f = b;
n = length( x )-1;
for k = 1:n
  for i = n+1:-1:k+1
    f(i) = f(i) - x(k)*f(i-1);
  end
end
for k = n:-1:1
  for i = k+1:n+1
    f(i) = f(i)/( x(i) - x(i-k) );
  end
  for i = k:n
    f(i) = f(i) - f(i+1);
  end
end

return 



function [alfa,beta]=vbnewtom(k,tloc)
%
%
% function che impone le condizioni d'ordine per TOM usando la forma di
% Bernstein
% Input:
% k    = numero di passi del metodo TOM considerato 
% tloc = vettore di k+1 componenti contenente k+1 ascisse successive della
% mesh
%
  p=2*k;              % ordine del metodo
  k1=ceil(k/2);
  k2=floor(k/2);
%
% calcolo della matrice B (p+1)x(k+1) di collocazione dei Bernstein
%
  h=diff(tloc); 
  Htot=sum(h);
  x=[0 cumsum(h)/Htot]; 
  x(k+1)=1;
   for i=1:p+1
      B(i,1:k+1)=nchoosek(p,i-1)*x.^(i-1).*(1-x).^(p+1-i);
   end
%  
% calcolo della matrice Bd che colloca le derivate dei Bernstein
%
Bd= zeros(p+1,k+1);
Bd(1,:)=-p*B(1,:)-B(2,:);
 for i=2:p
    Bd(i,:)=(p+2-i)*B(i-1,:)+(2*i-2-p)*B(i,:)-i*B(i+1,:);
 end
Bd(p+1,:)=B(p,:)+p*B(p+1,:);
%
% calcolo dei coefficienti
%
M=[B -h(k1)/Htot*Bd;zeros(1,k+1) ones(1,k+1)];
sol=M\[zeros(2*k+1,1);1];
alfa=sol(1:k+1); beta=sol(k+2:2*k+2);
  
return  



function [beta]=vbnewgam(k,tloc,ji)
%    
  p=k+1;        % ordine del metodo
  k1=ceil(k/2);
  k2=floor(k/2);
%
% calcolo della matrice B (p+1)x(k+1) di collocazione dei Bernstein
%
  h=diff(tloc); 
  Htot=sum(h);
  x=[0 cumsum(h)/Htot]; 
  x(k+1)=1;
  for i=1:p+1
      B(i,1:k+1)=nchoosek(p,i-1)*x.^(i-1).*(1-x).^(p+1-i);
  end
%
% calcolo della matrice Bd che colloca le derivate dei Bernstein
%
Bd= zeros(p+1,k+1);
Bd(1,:)=-p*B(1,:)-B(2,:);
 for i=2:p
    Bd(i,:)=(p+2-i)*B(i-1,:)+(2*i-2-p)*B(i,:)-i*B(i+1,:);
 end
%
% calcolo dei coefficienti
%
M=[B -h(k1)/Htot*Bd;zeros(1,k+1) ones(1,k+1)];  
b1=zeros(k+1,1);
 if ji > 0
    ind = ji;
    b1(ji)=-1;
    b1(ji+1)=1;
 else
     ind=1;
     b1(ji+1)=1;
     b1(ji+2)=-1;
end
b1= h(ind)/Htot*Bd(1:k+1,1:k+1)*b1;
M = B(1:k+1,1:k+1); 
sol=M\b1;
beta=sol;
  
return  



function beta=vbnew_gam(k,tloc,J)
%
% function che impone le condizioni d'ordine per GAM usando la forma di
% Bernstein
% Input:
% k    = numero di passi del metodo GAM considerato 
% tloc = vettore di k+1 componenti contenente k+1 ascisse successive della
% mesh
% J    = intero >= -k_1+1 e <=k_2 mediante il quale si specifica il
%        particolare metodo GAM desiderato (J=0 per il metodo principale)
%
%  y_{n+J}-y_{n+J-1} = 
%  h_{n+J} \sum_{i=-k_1}^{k_2} beta_{i+k_1} f_{n+i}
%
  p=k+1;            % ordine del metodo
  k1=ceil(k/2);
  k2=floor(k/2);
%
% calcolo della matrice B (p+1)x(k+1) di collocazione dei Bernstein
%
h=diff(tloc); Htot=sum(h);
x=[0 cumsum(h)/Htot]; x(k+1)=1;
  for i=1:p+1
      B(i,1:k+1)=nchoosek(p,i-1)*x.^(i-1).*(1-x).^(p+1-i);
  end
%
% calcolo della matrice Bd che colloca le derivate dei Bernstein
%
Bd= zeros(p+1,k+1);
Bd(1,:)=-p*B(1,:)-B(2,:);
 for i=2:p
    Bd(i,:)=(p+2-i)*B(i-1,:)+(2*i-2-p)*B(i,:)-i*B(i+1,:);
 end
Bd(p+1,:)=B(p,:)+p*B(p+1,:);
% 
%  calcolo dell'indice della colonna di B corrispondente a t_{n+J}
   ic=J+k1+1;
   tn=Htot/h(k1+J)*( B(:,ic)-B(:,ic-1) );
%
%  devo eliminare dal sistema lineare una riga se k ? dispari (quella
%  centrale) e due righe e sostituirle con la loro differenza se k ? pari
%
   if floor(k/2)*2==k                    % k pari
      M=[Bd(1:k/2-1,:); Bd(k/2,:)-Bd(k/2+1,:);Bd(k/2+2:k+2,:)]; 
      tn=[tn(1:k/2-1); tn(k/2)-tn(k/2+1); tn(k/2+2:k+2)]; 
   else
     M=[Bd(1:(k+1)/2,:);Bd((k+1)/2+2:k+2,:)];
     tn=[tn(1:(k+1)/2);tn((k+1)/2+2:k+2)];
   end
% calcolo dei coefficienti
%
beta=M\tn;

return



function x=estendinodi(nodi,N,j)
%
% x=estendinodi(nodi,N,j)
%
% Prolunga i nodi di input per blocco N con punti interni equidistanti
%

r=length(nodi)-1;

 if (N<r), 
    error('Dati non corretti. Deve essere N>=length(nodi)-1'), 
 end
 if (nargin<3), 
     j=r; 
 end
 if (j>r | j<=0), 
     error('Dati non corretti. Deve essere 0<j<=length(nodi)-1'), 
 end
nodi=nodi(:).';
hi=diff(nodi);
[hm,im]=max(hi);
h=sort(hi);hm=h(j);
im=fix((r+1)/2);
h=[hi(1:im) hm*ones(1,N-r) hi(im+1:r)];
x=[0 cumsum(h)];

return
  


function [x,w,P]=lglnodes(N)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% lglnodes.m
%
% Computes the Legendre-Gauss-Lobatto nodes, weights and the LGL Vandermonde 
% matrix. The LGL nodes are the zeros of (1-x^2)*P'_N(x). Useful for numerical
% integration and spectral methods. 
%
% Reference on LGL nodes and weights: 
%   C. Canuto, M. Y. Hussaini, A. Quarteroni, T. A. Tang, "Spectral Methods
%   in Fluid Dynamics," Section 2.3. Springer-Verlag 1987
%
% Written by Greg von Winckel - 04/17/2004
% Contact: gregvw@chtm.unm.edu
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Truncation + 1
N1=N+1;

% Use the Chebyshev-Gauss-Lobatto nodes as the first guess
x=cos(pi*(0:N)/N)';

% The Legendre Vandermonde Matrix
P=zeros(N1,N1);

% Compute P_(N) using the recursion relation
% Compute its first and second derivatives and 
% update x using the Newton-Raphson method.

xold=2;

while max(abs(x-xold))>eps
    xold=x;        
    P(:,1)=1;    P(:,2)=x;
    for k=2:N
        P(:,k+1)=( (2*k-1)*x.*P(:,k)-(k-1)*P(:,k-1) )/k;
    end
    x=xold-( x.*P(:,N1)-P(:,N) )./( N1*P(:,N1) );             
end
w=2./(N*N1*P(:,N1).^2); 

return