% English:
% ========
% DOP853MEX: Interface for Dopri853
% [tNodes,xNodes,stats,taupred]=dop853Mex(f,t,x0,opt)
%
% Input:
%   f       right side: Either the name of a function or
%           a function handle or an inline function
%           Function must be declared as
%             function dx=f(t,x)
%           (also see "FuncCallMethod" option)
%   t       row (double) vector containing the t nodes where the
%           solution has to be evaluated. Minimal requirement: 
%           two components (Start- and End-value). If t has more than two
%           components then dense output will be enabled.
%   x0      (d,1) double-Startvector (d dimension)
%   opt     optional struct with options; see below 
%
% Output:
%   tNodes  evaluated t nodes 
%   xNodes  solution at nodes in t 
%   stats   optional (1,5) vector for statistics:
%           1. IDID of DOPRI 
%              1 successfull integration 
%              2 successfull integration, early abort because of OutputFunction 
%             <0 integration failed
%             -2 reached max. number of steps 
%             -3 step size too small
%             -4 problem probably stiff
%           2. number of f-evaluations 
%           3. number of calculated steps 
%           4. number of accepted steps 
%           5. number of discarded steps 
%   taupred optional scalar for the next step size prediction
%
% Options: (Optionen:)
% Options:
% (in round braces: defaults)   
% [in square brackets: parameter name in DOPRI] 
% 
% If a option is needed and not found in the opt struct (or even the opt struct
% is missing), then the default-value in braces are used.
% 
%  * RelTol (1e-3) [RTOL]
%      relative tolerance 
%      either RelTol and AbsTol are both scalars or both are
%      (d,1) double vectors with RelTol and AbsTol for each
%      component.
%  * AbsTol (1e-6) [ATOL]
%      absolute tolerance 
%      either RelTol and AbsTol are both scalars or both are
%      (d,1) double vectors with RelTol and AbsTol for each
%      component.
%  * eps (2.3e-16) [WORK(1)]
%      rounding unit
%  * rho (0.9) [WORK(2)]
%      safety factor in step size prediction 
%  * StepSizeMinSelection (0.33) [WORK(3)]
%      Lower bound  for step size prediciton
%        StepSizeMinSelection <= tauNew/tauOld
%  * StepSizeMaxSelection (6.0) [WORK(4)]
%      Upper bound  for step size prediciton
%         tauNew/tauOld <= StepSizeMaxSelection
%  * StepSizeBeta (0.0) [WORK(5)]
%      Stabilizing factor for step size prediciton, cf.
%      Hairer,Wanner: Solving ODE II, Sect. IV.2 (SC-Stability)
%      0<=StepSizeBeta<=0.2
%  * MaxStep (tEnd-tStart) [WORK(6)]
%      maximal step size 
%  * InitialStep (0.0) [WORK(7)]
%      initial step size; if InitialStep=0.0, then the initial step
%      size will be calculated/estimated
%  * MaxNumberOfSteps (100000) [IWORK(1)]
%      maximal number of steps 
%  * StiffTestAfterStep (1000) [IWORK(4)]
%      Do stiffness test after StiffTestAfterStep steps
%
% --- Below are the options for the MEX-File; they do not belong to DOPRI
%
%  * IncludeGridPointsInDenseOutput (0) 
%      if t has more than two components, dense output will be
%      enabled to make it possible to evaluate the solution at the
%      given t nodes. For the output vectors (tNodes,xNodes) there
%      are two possibilites:
%      IncludeGridPointsInDenseOutput is 0, then (tNodes,xNodes) consists
%      only of the evaluted solution at the given nodes.
%      If IncludeGridPointsInDenseOutput is not 0, then also the
%      additionally generated nodes are included.
%  * OutputFcn (<none>)
%      Output Function (name, handle or inline)
%      Let the name of the output function be outfcn. At the
%      beginning of the integration outfcn is called:
%      outfcn([tStart,tEnd],x0,'init')
%      After every successfull integration step outfcn is called:
%      status=outfcn(t,x) 
%      if outfcn returns status=1, then the integration is stopped/aborted
%      (even if tEnd were not reached).
%      At the end of the integration outfcn is called:
%      outfcn([],[],'done')
%      There are already output functions available in Matlab:
%      cf: odeplot, odephas2, odephas3, odeprint
%  * OutputCallMode (1)
%      only relevant if dense output is enabled and an output function is given.
%      OutputCallMode defines when outfcn got called:
%        1 at the nodes given in t
%        2 at the nodes generated by DOPRI
%        3 case 1 and 2
%  * FuncCallMethod (1)
%      option to indicate, which method to use for calling external functions 
%      (like the right side or the output function) 
%        0 call maxCALLMATLAB direct
%          then ONLY names (strings) can be passed for functions
%        1 use maxCALLMATLAB to call feval
%          then ALSO function handles and inline-functions can be used
%  * OptWarnMiss (0)
%      Flag to warn if option is missing 
%        1 Warn if needed option in opt-struct is missing
%        0 don't warn
%  * OptWarnType (1)
%      Flag to warn if wrong type is found in options 
%        1 warn if a entry with wrong type is found in opt struct
%        0 don't warn
%  * OptWarnSize (1)
%      Flag to warn if option value has wrong size 
%        1 warn if a option value has wrong size 
%        0 don't warn
%
% The opt struct can be generated with odeset, although there are
% some additional entries and some other entries are missing. With
% odeset ONLY the following parameters can be set:
%   AbsTol,InitialStep,MaxStep,OutputFcn,RelTol
%
%  IMPORTANT NOTICE (if the right side is also a MEX-File)
%     the right side must not "garble" the passed mxArrays:
%     the size must not be changed and the memory must not
%     be freed. 
%     Hence: the right side has to take care, that the
%     passed mxArrays have the same size and enough memory
%     when returning. The values in the memory(-block)
%     may be overwritten.
%     If the right side is an m-file MATLAB obeys this
%     restriction automatically.
%
%
%
% German:
% =======
% DOP853MEX: Interface zu Dopri853
% [tNodes,xNodes,stats,taupred]=dop853Mex(f,t,x0,opt)
% 
% Input:
%   f       rechte Seite: entweder Funktionsname oder Funktions-Handle
%           oder inline-Funktion
%           Funktion muss folgende Form haben:
%             function dx=f(t,x)
%           (siehe in diesem Zusammenhang "FuncCallMethod" Option)
%   t       Zeilenvektor mit auszuwertenden t Stellen, muss mindestens 
%           2 Werte (Start- und Endwert) enthalten. Bei mehr als zwei Werten
%           wird auf dense-Output umgeschaltet
%   x0      (d,1) double-Startvektor (d Dimensionen)
%   opt     optionale struct mit Optionen: vgl. Beschreibung unten
%
% Output:
%   tNodes  ausgewertete t Zeitpunkte
%   xNodes  Lsung fr Zeitpunkte tNodes
%   stats   optionaler (1,5)-Vektor mit Statistik:
%           1. IDID von DOPRI
%              1 Erfolgreiche Integration
%              2 Erfolgreiche Integration, vorz. Abbruch durch OutputFunction
%             <0 erfolglose Integration
%             -2 max. Anzahl der Schritte wurde erreicht
%             -3 Schrittweite wurde zu klein
%             -4 Problem wahrscheinlich steif
%           2. Anzahl der f-Auswertungen
%           3. Anzahl der berechneten Schritte
%           4. Anzahl der akzeptierten Schritte
%           5. Anzahl der verworfenen Schritte
%   taupred optionaler Skalar mit dem nchsten Schrittweitenvorschlag
%
% Optionen:
% (in round braces: defaults)   
% [in eckigen Klammern: entsprechender Name bei DOPRI]
% 
% Falls eine Option bentigt wird und nicht in der opt-struct vorhanden ist 
% (oder die opt-struct selbst fehlt), so wird der in Klammern angegebene
% default-Wert verwendet.
% 
%  * RelTol (1e-3) [RTOL]
%      relative Toleranz
%      entweder sind RelTol und AbsTol beide Skalare oder beide 
%      (d,1)-double-Vektoren fr die einzelnen Zustands-Komponenten
%  * AbsTol (1e-6) [ATOL]
%      absolute Toleranz
%      entweder sind RelTol und AbsTol beide Skalare oder beide 
%      (d,1)-double-Vektoren fr die einzelnen Zustands-Komponenten
%  * eps (2.3e-16) [WORK(1)]
%      Maschinengenauigkeit
%  * rho (0.9) [WORK(2)]
%      Sicherheitsfaktor bei der Schrittweitensteuerung
%  * StepSizeMinSelection (0.33) [WORK(3)]
%      Untere Grenze bei der Schrittweitensteuerung
%        StepSizeMinSelection <= tauNew/tauOld
%  * StepSizeMaxSelection (6.0) [WORK(4)]
%      Obere Grenze bei der Schrittweitensteuerung
%         tauNew/tauOld <= StepSizeMaxSelection
%  * StepSizeBeta (0.0) [WORK(5)]
%      Stabilisierungsfaktor fr Schrittweitensteuerung vgl. 
%      Hairer,Wanner: Solving ODE II, Sect. IV.2 (SC-Stabilitt)
%      0<=StepSizeBeta<=0.2
%  * MaxStep (tEnd-tStart) [WORK(6)]
%      maximale Schrittweite
%  * InitialStep (0.0) [WORK(7)]
%      Startschrittweite, falls InitialStep=0.0 ist, so wird die 
%      Startschrittweite berechnet/geschtzt
%  * MaxNumberOfSteps (100000) [IWORK(1)]
%      maximale Anzahl von Schritten
%  * StiffTestAfterStep (1000) [IWORK(4)]
%      Test fr Steifheit aktivieren nach jeweils StiffTestAfterStep 
%      Schritten
% 
% --- Ab jetzt kommen Optionen, die zum MEX-File gehren,
% --- nicht mehr zu DOPRI.
%
%  * IncludeGridPointsInDenseOutput (0) 
%      falls t mehr als zwei Werte enthlt, wird auf dense-Output 
%      umgeschaltet, damit an den angegeben Stellen ausgewertet werden kann.
%      Fr die Ausgabe (tNodes,xNodes) gibt es nun zwei Mglichkeiten: 
%      IncludeGridPointsInDenseOutput ist 0, dann enthlt (tNodes,xNodes) 
%      nur die Ausgabe an den vorgegebenen Stellen. Ist  
%      IncludeGridPointsInDenseOutput verschieden von 0, so werden 
%      zustzlich auch noch die von DOPRI generierten Gitterpunkte 
%      ausgegeben.
%  * OutputFcn (<keine>)
%      Output Funktion (Name oder Handle oder Inline)
%      Sei der Name der OutputFunction outfcn. Dann wird zu Beginn der 
%      Integration aufgerufen: 
%      outfcn([tStart,tEnd],x0,'init'). 
%      Nach jedem erfolgreichen Integrationsschritt wird aufgerufen: 
%      status=outfcn(t,x).
%      Liefert outfcn einen status=1, so wird die Integration abgebrochen 
%      (auch wenn tEnd noch nicht erreicht). Sonst geht es weiter. Am Ende 
%      der Integration wird aufgerufen: 
%      outfcn([],[],'done')
%      Es gibt schon vorgefertigte OutputFunctions in Matlab:
%      vgl.: odeplot, odephas2, odephas3, odeprint
%  * OutputCallMode (1)
%      nur bei dense-Output mit OutputFunction interessant. Bestimmt wann 
%      outfcn aufgerufen wird:
%        1 bei den in t angegeben Stellen
%        2 bei den von DOPRI erstellen Gitterpunkten
%        3 sowohl 1 und 2
%  * FuncCallMethod (1)
%      gibt an, welche Methode zum Aufruf "externer" Funktionen (z.B.
%      rechte Seite oder Output-Funktion) verwendet werden soll:
%        0 direkt mexCallMATLAB verwenden 
%          dann knnen NUR Funktionsnamen bergeben werden
%        1 feval mit mexCALLMATLAB verwenden
%          dann knnen AUCH Funktions-Handle und Inline-Funktionen
%          bergeben werden.
%  * OptWarnMiss (0)
%      Warnungsflag bei fehlenden Optionen
%        1 Warnung, wenn in opt-struct eine bentigte Option fehlt
%        0 keine Warnung
%  * OptWarnType (1)
%      Warnungsflag bei Optionen mit falschem Typ
%        1 Warnung, wenn in opt-struct ein Eintrag den falschen Datentyp hat
%        0 keine Warnung
%  * OptWarnSize (1)
%      Warnungsflag bei Optionen mit falscher Gre
%        1 Warnung, wenn in opt-struct ein Eintrag die falsche Gre hat
%        0 keine Warnung
% 
% opt kann auch mit odeset generiert werden, obwohl dort dann  mehr 
% Felder generiert werden, als bentigt und einige Optionen von hier 
% nicht untersttzt werden. Also kann man mit odeset NUR einstellen:
%   AbsTol,InitialStep,MaxStep,OutputFcn,RelTol
%
%  WICHTIGE Anmerkung (falls rechte Seite "f" auch MEX-File ist)
%     die rechte Seite darf die bergebenen mxArrays nicht "verstmmeln": 
%     die Gre darf nicht verndert werden und auch der Speicherplatz 
%     darf nicht freigegeben werden. Also: die rechte Seite muss 
%     sicherstellen, dass die bergebenen mxArrays am Ende wieder 
%     diesselbe Gre und ausreichend Speicher haben. Der Speicher 
%     selbst darf natrlich zu rechenzwecken berschrieben werden.
%     Bei m-Files muss man darauf keine Rcksicht nehmen, denn da achtet 
%     MATLAB automatisch darauf.
%
